/*******************************************************************************
 * Copyright (c) 2003, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     Sian Whiting - initial version
 *******************************************************************************/
package org.eclipse.contribution.visualiser.utils;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.RGB;

/**
 * Utility class which markups providers can use to get colours
 * Could be extended to add methods like getPastelColors, getGreys, 
 * getListOfNRandomColours, etc.. (as needed)
 */
public class ColorUtils {
	
	// Colour constants
	private static Color red1 = new Color(null, new RGB(255,128,128));
	private static Color yellow1 = new Color(null, new RGB(255,255,128));
	private static Color green1 = new Color(null, new RGB(128,255,128));
	private static Color cyan1 = new Color(null, new RGB(128,255,255));
	private static Color blue1 = new Color(null, new RGB(128,128,255));
	private static Color purple1 = new Color(null, new RGB(255,128,255));
	private static Color orange1 = new Color(null, new RGB(255,192,128));
	private static Color swamp1 = new Color(null, new RGB(192,255,128));
	private static Color mint1 = new Color(null, new RGB(128,255,192));
	private static Color lightblue1 = new Color(null, new RGB(128,192,255));
	private static Color indigo1 = new Color(null, new RGB(192,128,255));
	private static Color pink1 = new Color(null, new RGB(255,128,192));
	
	private static Color red2 = new Color(null, new RGB(255,64,64));
	private static Color yellow2 = new Color(null, new RGB(255,255,64));
	private static Color green2 = new Color(null, new RGB(64,255,64));
	private static Color cyan2 = new Color(null, new RGB(64,255,255));
	private static Color blue2 = new Color(null, new RGB(64,64,255));
	private static Color purple2 = new Color(null, new RGB(255,64,255));
	private static Color orange2 = new Color(null, new RGB(255,170,64));
	private static Color swamp2 = new Color(null, new RGB(170,255,64));
	private static Color mint2 = new Color(null, new RGB(64,255,170));
	private static Color lightblue2 = new Color(null, new RGB(64,170,255));
	private static Color indigo2 = new Color(null, new RGB(170,64,255));
	private static Color pink2 = new Color(null, new RGB(255,64,170));	
	
	private static Color red3 = new Color(null, new RGB(255,0,0));
	private static Color yellow3 = new Color(null, new RGB(255,255,0));
	private static Color green3 = new Color(null, new RGB(0,255,0));
	private static Color cyan3 = new Color(null, new RGB(0,255,255));
	private static Color blue3 = new Color(null, new RGB(0,0,255));
	private static Color purple3 = new Color(null, new RGB(255,0,255));
	private static Color orange3 = new Color(null, new RGB(255,128,0));
	private static Color swamp3 = new Color(null, new RGB(128,255,0));
	private static Color mint3 = new Color(null, new RGB(0,255,128));
	private static Color lightblue3 = new Color(null, new RGB(0,128,255));
	private static Color indigo3 = new Color(null, new RGB(128,0,255));
	private static Color pink3 = new Color(null, new RGB(255,0,128));		
	
	private static Color red4 = new Color(null, new RGB(192,0,0));
	private static Color yellow4 = new Color(null, new RGB(192,192,0));
	private static Color green4 = new Color(null, new RGB(0,192,0));
	private static Color cyan4 = new Color(null, new RGB(0,192,192));
	private static Color blue4 = new Color(null, new RGB(0,0,192));
	private static Color purple4 = new Color(null, new RGB(192,0,192));
	private static Color orange4 = new Color(null, new RGB(192,100,0));
	private static Color swamp4 = new Color(null, new RGB(100,192,0));
	private static Color mint4 = new Color(null, new RGB(0,192,100));
	private static Color lightblue4 = new Color(null, new RGB(0,100,192));
	private static Color indigo4 = new Color(null, new RGB(100,0,192));
	private static Color pink4 = new Color(null, new RGB(192,0,100));	
	
	private static Color red5 = new Color(null, new RGB(128,0,0));
	private static Color yellow5 = new Color(null, new RGB(128,128,0));
	private static Color green5 = new Color(null, new RGB(0,128,0));
	private static Color cyan5 = new Color(null, new RGB(0,128,128));
	private static Color blue5 = new Color(null, new RGB(0,0,128));
	private static Color purple5 = new Color(null, new RGB(128,0,128));
	private static Color orange5 = new Color(null, new RGB(128,64,0));
	private static Color swamp5 = new Color(null, new RGB(64,128,0));
	private static Color mint5 = new Color(null, new RGB(0,128,64));
	private static Color lightblue5 = new Color(null, new RGB(0,64,128));
	private static Color indigo5 = new Color(null, new RGB(64,0,128));
	private static Color pink5 = new Color(null, new RGB(128,0,64));
	
//	private static Color red6 = new Color(null, new RGB(64,0,0));
//	private static Color yellow6 = new Color(null, new RGB(64,64,0));
//	private static Color green6 = new Color(null, new RGB(0,64,0));
//	private static Color cyan6 = new Color(null, new RGB(0,64,64));
//	private static Color blue6 = new Color(null, new RGB(0,0,64));
//	private static Color purple6 = new Color(null, new RGB(64,0,64));

	private static List colorList; 

	
	/**
	 * Get the list of all defined Colors
	 * @return List of Colors
	 */
	public static List getColorList(){
		if(colorList==null){
			colorList = new ArrayList();

			colorList.add(cyan2);
			colorList.add(blue2);
			colorList.add(purple2);
			colorList.add(swamp2);
			colorList.add(orange2);
			colorList.add(indigo2);
			colorList.add(lightblue2);
			colorList.add(red2);
			colorList.add(yellow2);
			colorList.add(green2);
			colorList.add(pink2);
			colorList.add(mint2);

			colorList.add(red4);
			colorList.add(yellow4);
			colorList.add(green4);
			colorList.add(cyan4);
			colorList.add(blue4);
			colorList.add(purple4);
			colorList.add(orange4);
			colorList.add(swamp4);
			colorList.add(mint4);
			colorList.add(lightblue4);
			colorList.add(indigo4);
			colorList.add(pink4);

			colorList.add(red3);
			colorList.add(yellow3);
			colorList.add(green3);
			colorList.add(cyan3);
			colorList.add(blue3);
			colorList.add(purple3);
			colorList.add(orange3);
			colorList.add(swamp3);
			colorList.add(mint3);
			colorList.add(lightblue3);
			colorList.add(indigo3);
			colorList.add(pink3);

			colorList.add(red1);
			colorList.add(yellow1);
			colorList.add(green1);
			colorList.add(cyan1);
			colorList.add(blue1);
			colorList.add(purple1);
			colorList.add(orange1);
			colorList.add(swamp1);
			colorList.add(mint1);
			colorList.add(lightblue1);
			colorList.add(indigo1);
			colorList.add(pink1);
						
			colorList.add(red5);
			colorList.add(yellow5);
			colorList.add(green5);
			colorList.add(cyan5);
			colorList.add(blue5);
			colorList.add(purple5);
			colorList.add(orange5);
			colorList.add(swamp5);
			colorList.add(mint5);
			colorList.add(lightblue5);
			colorList.add(indigo5);
			colorList.add(pink5);


		}
		return colorList;
	}
	
	
	/**
	 * Returns a colour with random R,G and B values that are between 50 and 250 
	 * and are multiples of 5.
	 * @return randomly generated Color
	 */
	public static Color getRandomColor(){
		int r =  (int)(((int)(Math.random()*(double)40))*5+50);
		int g = (int)(((int)(Math.random()*(double)40))*5+50);
		int b = (int)(((int)(Math.random()*(double)40))*5+50);
		return new Color(null, new RGB(r,g,b));
	}

}
